Package: edgeR
Version: 4.11.1
Date: 2026-05-23
Title: Empirical Analysis of Digital Gene Expression Data in R
Description: Differential expression analysis of sequence count data. Implements a range of statistical methodology based on the negative binomial distributions, including empirical Bayes estimation, exact tests, generalized linear models, quasi-likelihood, and gene set enrichment. Can perform differential analyses of any type of omics data that produces read counts, including RNA-seq, ChIP-seq, ATAC-seq, Bisulfite-seq, SAGE, CAGE, metabolomics, or proteomics spectral counts. RNA-seq analyses can be conducted at the gene or isoform level, and tests can be conducted for differential exon or transcript usage.
Author: Yunshun Chen, Lizhong Chen, Aaron TL Lun, Davis J McCarthy, Pedro Baldoni, Matthew E Ritchie, Belinda Phipson, Yifang Hu, Xiaobei Zhou, Mark D Robinson, Gordon K Smyth
Maintainer: Yunshun Chen <yuchen@wehi.edu.au>, Gordon Smyth <smyth@wehi.edu.au>,
 Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>,
 Mark Robinson <mark.robinson@imls.uzh.ch>
License: GPL (>=2)
Depends: R (>= 3.6.0), limma (>= 3.63.6)
Imports: methods, graphics, stats, utils, locfit
Suggests: jsonlite, knitr, Matrix, nanoparquet, readr, rhdf5,
        SeuratObject, splines, AnnotationDbi, Biobase, BiocStyle,
        org.Hs.eg.db, S4Vectors, SummarizedExperiment
VignetteBuilder: knitr
URL: https://bioinf.wehi.edu.au/edgeR/,
        https://bioconductor.org/packages/edgeR
biocViews: AlternativeSplicing, BatchEffect, Bayesian,
        BiomedicalInformatics, CellBiology, ChIPSeq, Clustering,
        Coverage, DifferentialExpression, DifferentialMethylation,
        DifferentialSplicing, DNAMethylation, Epigenetics,
        FunctionalGenomics, GeneExpression, GeneSetEnrichment,
        Genetics, Genetics, ImmunoOncology, MultipleComparison,
        Normalization, Pathways, Proteomics, QualityControl,
        Regression, RNASeq, SAGE, Sequencing, SingleCell,
        SystemsBiology, TimeCourse, Transcription, Transcriptomics
NeedsCompilation: yes
git_url: https://git.bioconductor.org/packages/edgeR
git_branch: devel
git_last_commit: ae520cd
git_last_commit_date: 2026-05-22
Repository: Bioconductor 3.24
Date/Publication: 2026-05-24
Packaged: 2026-05-24 20:20:00 UTC; biocbuild
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-25 02:44:43 UTC; unix
