Package: edgeR
Version: 4.0.3
Date: 2023-12-09
Title: Empirical Analysis of Digital Gene Expression Data in R
Description: Differential expression analysis of RNA-seq expression profiles with biological replication. Implements a range of statistical methodology based on the negative binomial distributions, including empirical Bayes estimation, exact tests, generalized linear models and quasi-likelihood tests. As well as RNA-seq, it be applied to differential signal analysis of other types of genomic data that produce read counts, including ChIP-seq, ATAC-seq, Bisulfite-seq, SAGE and CAGE.
Author: Yunshun Chen, Aaron TL Lun, Davis J McCarthy, Lizhong Chen, Matthew E Ritchie, Belinda Phipson, Yifang Hu, Xiaobei Zhou, Mark D Robinson, Gordon K Smyth
Maintainer: Yunshun Chen <yuchen@wehi.edu.au>, Gordon Smyth <smyth@wehi.edu.au>, Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>, Mark Robinson <mark.robinson@imls.uzh.ch>
License: GPL (>=2)
Depends: R (>= 3.6.0), limma (>= 3.41.5)
Imports: methods, graphics, stats, utils, locfit, Rcpp
Suggests: jsonlite, readr, rhdf5, splines, knitr, AnnotationDbi,
        Biobase, BiocStyle, SummarizedExperiment, org.Hs.eg.db, Matrix,
        SeuratObject
LinkingTo: Rcpp
VignetteBuilder: knitr
URL: https://bioinf.wehi.edu.au/edgeR/,
        https://bioconductor.org/packages/edgeR
biocViews: GeneExpression, Transcription, AlternativeSplicing,
        Coverage, DifferentialExpression, DifferentialSplicing,
        DifferentialMethylation, GeneSetEnrichment, Pathways, Genetics,
        DNAMethylation, Bayesian, Clustering, ChIPSeq, Regression,
        TimeCourse, Sequencing, RNASeq, BatchEffect, SAGE,
        Normalization, QualityControl, MultipleComparison,
        BiomedicalInformatics, CellBiology, FunctionalGenomics,
        Epigenetics, Genetics, ImmunoOncology, SystemsBiology,
        Transcriptomics, SingleCell
NeedsCompilation: yes
git_url: https://git.bioconductor.org/packages/edgeR
git_branch: RELEASE_3_18
git_last_commit: 482cd3c
git_last_commit_date: 2023-12-09
Repository: Bioconductor 3.18
Date/Publication: 2023-12-10
Packaged: 2023-12-10 22:06:22 UTC; biocbuild
